@@ -76,13 +76,5 @@ def attempt_download(file, repo='meituan/YOLOv6', release='0.3.0'):
                     tag = release
 
         file.parent.mkdir(parents=True, exist_ok=True)  # make parent dir (if required)
-        if name in assets:
-            url3 = 'https://drive.google.com/drive/folders/1EFQTEUeXWSFww0luse2jB9M1QNZQGwNl'  # backup gdrive mirror
-            safe_download(
-                file,
-                url=f'https://github.com/{repo}/releases/download/{tag}/{name}',
-                url2=f'https://storage.googleapis.com/{repo}/{tag}/{name}',  # backup url (optional)
-                min_bytes=1E5,
-                error_msg=f'{file} missing, try downloading from https://github.com/{repo}/releases/{tag} or {url3}')
     
     return str(file)
